C  This program computes various forms of adjusted probability values for
C  multiple testing.
C
C  Reference:
C
C  Wright, S. P. 1992. Adjusted P-values for simultaneous inference. 
C     Biometrics 48: 1005-1013.
C
C  Legendre, P. & L. Legendre. 1998. Numerical ecology, 2nd English edition. 
C     Elsevier Science BV, Amsterdam. See p. 18.
C
C                                              Pierre Legendre, May 1995
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
      Parameter (kmax=500)
      Integer irank(kmax)
      Real xmat(kmax,5),prob(kmax)
      Common k,prob
C
      call Lecture
      open(7,file='PValues.out',status='NEW')
C
      do 2 i=1,k
      xmat(i,1)=0.0
    2 irank(i)=1
      do 10 i=1,k
      temp=prob(i)
      do 10 j=1,k
      if(prob(j).lt.temp) irank(i)=irank(i)+1
   10 continue
      do 14 i=1,k
   14 xmat(irank(i),1)=prob(i)
      do 16 i=2,k
      if(xmat(i,1).eq.0.0) xmat(i,1)=xmat(i-1,1)
   16 continue
C Bonferroni adjustment, Adjusted prob., Holm's and Hochberg's adjustments
      do 20 i=1,k
      xmat(i,2)=xmat(i,1)*float(k)
      xmat(i,3)=xmat(i,1)*float(k-i+1)
      xmat(i,4)=xmat(i,3)
      xmat(i,5)=xmat(i,3)
   20 continue
      do 22 i=2,k
      if(xmat(i,4).lt.xmat(i-1,4)) xmat(i,4)=xmat(i-1,4)
   22 continue
      do 24 i=k,2,-1
      if(xmat(i,5).lt.xmat(i-1,5)) xmat(i-1,5)=xmat(i,5)
   24 continue   
C Print results
      write(7,100)
      do 50 i=1,k
      write(7,101) i,(xmat(i,j), j=1,5)
   50 continue
      write(*,*)
      write(*,*) "The results are written to file 'PValues.out'"
      write(*,*) 'End of the program.'
      stop
  100 format
     +(/'         Prob.  p(Bonf.)    adj.p   adj.Holm  adj.Hochberg'/)
  101 format(1x,i3,5f10.5)
      end

      Subroutine Lecture
      Parameter (kmax=200)
      Real X(kmax,5),prob(kmax)
      Common k,prob
      Character*79 ipfn1
C
C Name of the input file  
      write(*,*) 'Name of the input file?'
      write(*,*)
      read(*,'(A)') ipfn1
      open(10,file=ipfn1)
      write(*,*) 'How many values in that file ?'
      write(*,*)
      read(*,*) k
      if(k.gt.kmax) then
         write(*,102)
         stop 3
         endif
C Read in the probability values
      read(10,*) (prob(i), i=1,k)
      close(10)
      return
  102 format(' This problem is too large. Re-dimension the program.')
      end
